@@ -9,7 +9,9 @@
 //  Module:         Configuration utility
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017,
+// 2018, 2019
+// Guy Voncken
 //
 // This file is part of libguytools.
 //
@@ -69,13 +71,10 @@ static t_pcchar KEYWORD_SECTIONSTART = "SECTION";
 static t_pcchar KEYWORD_SECTIONEND   = "ENDSECTION";
 static t_pcchar KEYWORD_INCLUDE      = "INCLUDE";
 static t_pcchar KEYWORD_INCLUDE_OPT  = "INCLUDE_OPTIONAL";
-static t_pcchar KEYWORD_REMARK       = "REM";
 static t_pcchar KEYWORD_DEFINE       = "DEFINE";
 static t_pcchar KEYWORD_UNDEFINE     = "UNDEFINE";
-
-// static t_pcchar MINUS_LINE = "------------------------------------------------------------------------------"
-//                              "------------------------------------------------------------------------------"
-//                              "------------------------------------------------------------------------------";
+static t_pcchar KEYWORD_REMARK       = "REM";
+static t_char   KEYCHAR_REMARK       = '#';
 
 /* ------------ */
 /*  Memory IDs  */
@@ -127,14 +126,14 @@ typedef char t_ToolCfgSectionName[MAX_SECTIONNAME_LEN+1];
 
 typedef struct
 {
-   t_int            BufferLen;
-   t_int            ActLineLen;
-   t_int            ActLineNr;
-   t_pchar         pActLine;
-   t_pchar         pBuffer;
-   t_char           FileName      [MAX_FILENAME_LEN +1];
+   t_int                BufferLen;
+   t_int                ActLineLen;
+   t_int                ActLineNr;
+   t_pchar             pActLine;
+   t_pchar             pBuffer;
+   t_char               FileName      [MAX_FILENAME_LEN +1];
+   t_int                SectionNesting;
    t_ToolCfgSectionName SectionNameArr[MAX_SECTIONNAMES];     // These section names are only valid during the the scanning of one include file
-   t_int            SectionNesting;
    t_ToolCfgState       State;
 } t_ToolCfgContext, *t_pToolCfgContext;
 
@@ -162,7 +161,7 @@ typedef struct
 /*           Variables            */
 /* ------------------------------ */
 
-t_ToolCfgLocal  ToolCfgLocal;
+static t_ToolCfgLocal  ToolCfgLocal;
 
 
 /* ------------------------------ */
@@ -186,7 +185,7 @@ static APIRET ToolCfgMemAlloc (t_pvoid *ppMem, t_uint Size, t_uint Options, t_in
       rc = Options; // To prevent from compiler warning "unused..."
       rc = MemId;   // To prevent from compiler warning "unused..."
       *ppMem = malloc (Size);
-      if (*ppMem == NULL)
+      if (*ppMem == nullptr)
            rc = TOOLCFG_ERROR_MALLOC_FAILED;
       else rc = NO_ERROR;
    #else
@@ -254,10 +253,10 @@ static APIRET ToolCfgLogEntry (t_pcchar pFileName, t_pcchar pFunctionName, t_int
 static APIRET ToolCfgStdErrorHeader (t_pcchar pActCursor)
 {
    t_pToolCfgContext pCfgContext;
-   t_pchar       pEndOfLine;
-   t_pchar       pBuff;
-   t_int          Cursor;
-   t_int          LineLen;
+   t_pchar           pEndOfLine;
+   t_pchar           pBuff;
+   t_int              Cursor;
+   t_int              LineLen;
 
    pCfgContext = ToolCfgLocal.pActCfgContext;
    if (pCfgContext->FileName[0] == '\0')
@@ -291,15 +290,21 @@ static APIRET ToolCfgStdErrorHeader (t_pcchar pActCursor)
 
 static t_int ToolCfgStrCmpNoCase (const char *pStr1, const char *pStr2)
 {
+   if ((pStr1 == nullptr) || (pStr2 == nullptr))
+      return 1;
+      
    #ifdef _WIN32
-      return strcmpi    (pStr1, pStr2);
+      return (t_int) strcmpi    (pStr1, pStr2);
    #else
-      return strcasecmp (pStr1, pStr2);
+      return (t_int) strcasecmp (pStr1, pStr2);
    #endif
 }
 
 static t_int ToolCfgStrNCmpNoCase (const char *pStr1, const char *pStr2, t_int Len)
 {
+   if ((pStr1 == nullptr) || (pStr2 == nullptr))
+      return 1;
+      
    #ifdef _WIN32
       return strnicmp    (pStr1, pStr2, Len);
    #else
@@ -319,7 +324,7 @@ static APIRET ToolCfgCopyName (t_pcchar pSrc, t_pchar pDst, t_int DstLen)
 {
    t_int SrcLen;
 
-   if (pSrc == NULL)
+   if (pSrc == nullptr)
       pDst[0] = '\0';
    else
    {
@@ -346,7 +351,7 @@ static APIRET ToolCfgFindFirstWord (t_pcchar pActLine, const char **ppFirstWord,
    {
    }
    if ((LineLen <= 0) || (**ppFirstWord == CHAR_LF) || (**ppFirstWord == CHAR_CR))
-      *ppFirstWord = NULL;
+      *ppFirstWord = nullptr;
 
    *pRemLineLen = LineLen;
 
@@ -360,7 +365,7 @@ static APIRET ToolCfgCompareKeyWord (t_pcchar pTableKeyWord, t_pchar pCheckKeyWo
    t_int     CheckKeyLen;
    t_pchar  pCh;
 
-   pNextChar = NULL;
+   pNextChar = nullptr;
    *pEq      = FALSE;
 
    TableKeyLen = (t_int) strlen (pTableKeyWord);
@@ -574,7 +579,7 @@ static APIRET ToolCfgBuildHelpSet (t_pToolCfgDataDesc pCfgDataDesc, t_pchar pHel
 
    for (i=0; ; i++)
    {
-      if (pSetArray[i].pSetString == NULL) /* end of array? */
+      if (pSetArray[i].pSetString == nullptr) /* end of array? */
          break;
       CFG_CHK_APP (ToolCfgChkStrCpy (pHelpBuff, pSetArray[i].pSetString, pHelpBuffLen))
       if (pSetArray[i+1].pSetString)
@@ -633,7 +638,7 @@ static APIRET ToolCfgBuildTableHelp (t_pToolCfgTableDesc pCfgTableDesc,
    for (i=0;;i++)
    {
       pCfgDataDesc = &pCfgTableDesc->pDataDescArray[i];
-      if (pCfgDataDesc->pName == NULL)
+      if (pCfgDataDesc->pName == nullptr)
          break;
       NameLen  = (t_int)strlen (pCfgDataDesc->pName);
       CFG_CHK_APP (ToolCfgChkStrCpy (pHelpBuff1, pCfgDataDesc->pName, pHelpBuffLen))
@@ -683,7 +688,7 @@ static APIRET ToolCfgQueryFileInfo (t_pchar pFileName, t_pint pYear, t_pint pMon
    // Linux code
    // ----------
    struct stat FileInfo;
-   struct tm*  LastModified = NULL;
+   struct tm*  LastModified = nullptr;
 
    memset (&FileInfo, 0, sizeof(FileInfo));
 
@@ -783,12 +788,12 @@ static APIRET ToolCfgReadCfgFile (t_pToolCfgContext pCfgContext)
 
    /* Find size */
    pCfgFile = fopen (&pCfgContext->FileName[0], "rb");
-   if (pCfgFile == NULL)
+   if (pCfgFile == nullptr)
    {
       CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Could not open configuration file %s", &pCfgContext->FileName[0]))
       return TOOLCFG_ERROR_OPEN_FAILED;
    }
-   rc = setvbuf (pCfgFile, NULL, _IOFBF, 32768);
+   rc = setvbuf (pCfgFile, nullptr, _IOFBF, 32768);
    if (rc)
       return TOOLCFG_ERROR_SETBUF_FAILED;
 
@@ -830,7 +835,7 @@ static APIRET ToolCfgGetLineLen (t_pToolCfgContext pCfgContext)
    t_int    CopyLen;
    t_int    KeepRunning;
 
-   pLastBackslash = NULL;
+   pLastBackslash = nullptr;
    KeepRunning    = TRUE;
 
    pEol = pCfgContext->pActLine;
@@ -855,7 +860,7 @@ static APIRET ToolCfgGetLineLen (t_pToolCfgContext pCfgContext)
                           CopyLen = pCfgContext->pBuffer + pCfgContext->BufferLen - pEol;
                           memmove (pLastBackslash, pEol, CopyLen);
                           pEol = pLastBackslash;
-                          pLastBackslash = NULL;
+                          pLastBackslash = nullptr;
                           pCfgContext->ActLineNr++;
                        }
                        else
@@ -863,7 +868,7 @@ static APIRET ToolCfgGetLineLen (t_pToolCfgContext pCfgContext)
                        break;
          case '\0'   : KeepRunning = FALSE;
                        break;
-         default     : pLastBackslash = NULL;
+         default     : pLastBackslash = nullptr;
                        pEol++;
       }
    }
@@ -877,11 +882,15 @@ static APIRET ToolCfgSearchLine (t_pToolCfgContext pCfgContext)
    t_pchar pTmp;
 
    pTmp = pCfgContext->pActLine + pCfgContext->ActLineLen;
-   if (*pTmp != '\0')
+   if (pTmp[0] != '\0')
    {
-      if (((pTmp[0] == CHAR_LF) && (pTmp[1] == CHAR_CR)) ||
-          ((pTmp[1] == CHAR_LF) && (pTmp[0] == CHAR_CR)))
-           pTmp += 2;
+      if (pTmp[1] != '\0')
+      {
+         if (((pTmp[0] == CHAR_LF) && (pTmp[1] == CHAR_CR)) ||
+             ((pTmp[1] == CHAR_LF) && (pTmp[0] == CHAR_CR)))
+              pTmp += 2;
+         else pTmp += 1;
+      }
       else pTmp += 1;
    }
    pCfgContext->ActLineNr++;
@@ -937,14 +946,14 @@ static APIRET ToolCfgDropContext (t_pToolCfgContext pCfgContext)
    return NO_ERROR;
 }
 
-/* CfgGetNextLine is the core function of the configuration scanner. From the calling   */
-/* function's point of view, it simply reads the next line (or returns ppLine = NULL if */
-/* no more data can be found). Internally it manages the INCLUDE and SECTION statements */
-/* as well as trailing backslashes (for joining lines).                                 */
-static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
+/* CfgGetNextLine is the core function of the configuration scanner. From the calling      */
+/* function's point of view, it simply reads the next line (or returns ppLine = nullptr if */
+/* no more data can be found). Internally it manages the INCLUDE and SECTION statements    */
+/* as well as trailing backslashes (for joining lines).                                    */
+static APIRET ToolCfgGetNextLine0 (t_pchar *ppLine, t_pint pLineLen)
 {
    t_pToolCfgContext pContext;
-   t_pchar       pEol, pTmp, pKeyWord, pFirstWord;
+   t_pchar       pEol, pTmp, pStart, pKeyWord, pFirstWord;
    t_int          ParamLen;
    t_int          RemLineLen;
    char           TmpChar;
@@ -993,13 +1002,17 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
             break;
 
          case CFGSTATE_CHECKEOF:
-            pTmp = pContext->pActLine;
+            for (  pTmp = pContext->pActLine;
+                 (*pTmp == ' ') || (*pTmp == '\t');
+                   pTmp++)
+            {
+            }
             if ((*pTmp == '\0') || (*pTmp == CHAR_EOF))
             {
                if (pContext->SectionNesting != 0)
                {
-                  pContext->pActLine = NULL;
-                  CFG_CHK_APP (ToolCfgStdErrorHeader (NULL))
+                  pContext->pActLine = nullptr;
+                  CFG_CHK_APP (ToolCfgStdErrorHeader (nullptr))
                   CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Missing statement %s before end of file.", KEYWORD_SECTIONEND))
                   return TOOLCFG_ERROR_CONFIG_ERROR;
                }
@@ -1007,10 +1020,10 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                ToolCfgLocal.IncludeNestingLevel--;
                if (ToolCfgLocal.IncludeNestingLevel == -1) /* did we leave the last nesting level */
                {
-                  pTmp = NULL;
-                  pContext->pActLine   = NULL;
+                  pTmp = nullptr;
+                  pContext->pActLine   = nullptr;
                   pContext->ActLineLen = 0;
-                  pContext->State = CFGSTATE_COMPLETED;
+                  pContext->State      = CFGSTATE_COMPLETED;
                }
                else
                {
@@ -1029,6 +1042,8 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                    pKeyWord++)
             {
             }
+            if (pKeyWord == nullptr)                // Should not occur (has been checked in CFGSTATE_CHECKEOF)
+               return TOOLCFG_ERROR_UNEXPECTED_EOL; 
             ChkInclude    = ToolCfgStrNCmpNoCase (pKeyWord, KEYWORD_INCLUDE    , (t_int) strlen (KEYWORD_INCLUDE    )) == 0;
             ChkIncludeOpt = ToolCfgStrNCmpNoCase (pKeyWord, KEYWORD_INCLUDE_OPT, (t_int) strlen (KEYWORD_INCLUDE_OPT)) == 0;
             if (ChkInclude || ChkIncludeOpt)
@@ -1056,7 +1071,17 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                RemLineLen = strlen (pFirstWord);
 
                // Get the filename
-               CFG_CHK_APP (ToolCfgFindFirstWord  (pFirstWord + ParamLen, (const char **) &pFirstWord, RemLineLen, &RemLineLen))
+               pStart = pFirstWord;
+               CFG_CHK_APP (ToolCfgFindFirstWord  (pFirstWord, (const char **) &pFirstWord, RemLineLen, &RemLineLen))
+               if (pFirstWord == nullptr)
+               {
+                  ToolCfgLocal.IncludeNestingLevel--;
+                  ToolCfgLocal.pActCfgContext = &ToolCfgLocal.pCfgContextStack[ToolCfgLocal.IncludeNestingLevel];
+                  CFG_CHK_APP (ToolCfgStdErrorHeader (pStart))
+                  CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Filename missing"))
+                  CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "The correct syntax is: %s <FileName> [SectionName1] [SectionName2] ... [SectionName16]", KEYWORD_INCLUDE))
+                  return TOOLCFG_ERROR_CONFIG_ERROR;
+               }
                CFG_CHK_APP (ToolCfgGetParamLen    (pFirstWord, &ParamLen))
                ToolCfgStrMaxCpy (&pContext->FileName[0], pFirstWord, min(MAX_FILENAME_LEN, ParamLen+1));
                RemLineLen -= ParamLen;
@@ -1065,10 +1090,13 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                for (i=0; ;i++)
                {
                   CFG_CHK_APP (ToolCfgFindFirstWord  (pFirstWord + ParamLen, (const char **) &pFirstWord, RemLineLen, &RemLineLen))
-                  if (pFirstWord == NULL)
+                  if (pFirstWord == nullptr)
                      break;
                   if (i >= MAX_SECTIONNAMES)
                   {
+                     ToolCfgLocal.IncludeNestingLevel--;
+                     ToolCfgLocal.pActCfgContext = &ToolCfgLocal.pCfgContextStack[ToolCfgLocal.IncludeNestingLevel];
+                     CFG_CHK_APP (ToolCfgStdErrorHeader (pFirstWord + ParamLen))
                      CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Too many parameters for keyword %s or %s", KEYWORD_INCLUDE, KEYWORD_INCLUDE_OPT))
                      CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "The correct syntax is: %s <FileName> [SectionName1] [SectionName2] ... [SectionName16]", KEYWORD_INCLUDE))
                      return TOOLCFG_ERROR_CONFIG_ERROR;
@@ -1093,20 +1121,15 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                    pKeyWord++)
             {
             }
+            if (pKeyWord == nullptr)                // Should not occur (has been checked in CFGSTATE_CHECKEOF)
+               return TOOLCFG_ERROR_UNEXPECTED_EOL; 
             if (ToolCfgStrNCmpNoCase (pKeyWord, KEYWORD_SECTIONSTART, (t_int) strlen (KEYWORD_SECTIONSTART)) == 0)
             {
-//               if (pContext->SectionNesting >0)
-//               {
-//                  CFG_CHK_APP (ToolCfgStdErrorHeader (pKeyWord))
-//                  CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "SECTION statements are not allowed to be nested within one file."))
-//                  return TOOLCFG_ERROR_CONFIG_ERROR;
-//               }
-
                pFirstWord = pKeyWord + strlen (KEYWORD_SECTIONSTART);
                RemLineLen = strlen (pFirstWord);
 
                CFG_CHK_APP (ToolCfgFindFirstWord (pFirstWord, (const char **) &pFirstWord, RemLineLen, &RemLineLen))
-               if (pFirstWord == NULL)
+               if (pFirstWord == nullptr)
                {
                   CFG_CHK_APP (ToolCfgStdErrorHeader (pFirstWord))
                   CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Section name expected."))
@@ -1114,7 +1137,7 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                   return TOOLCFG_ERROR_CONFIG_ERROR;
                }
                SectionNameFound = FALSE;
-               while (pFirstWord != NULL)
+               while (pFirstWord != nullptr)
                {
                   CFG_CHK_APP (ToolCfgGetParamLen (pFirstWord, &ParamLen))
                   TmpChar = pFirstWord[ParamLen];
@@ -1177,7 +1200,6 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
             break;
 
          case CFGSTATE_SEARCHSECTIONEND:
-
             KeyLenSectionStart = (t_int) strlen (KEYWORD_SECTIONSTART);
             KeyLenSectionEnd   = (t_int) strlen (KEYWORD_SECTIONEND  );
             for (SectionSubNesting = 0;;)
@@ -1185,8 +1207,8 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
                CFG_CHK_APP (ToolCfgSearchLine (pContext))
                if (*pContext->pActLine == '\0')
                {
-                  pContext->pActLine = NULL;
-                  CFG_CHK_APP (ToolCfgStdErrorHeader (NULL))
+                  pContext->pActLine = nullptr;
+                  CFG_CHK_APP (ToolCfgStdErrorHeader (nullptr))
                   CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Missing statement ENDSECTION before end of file."))
                   return TOOLCFG_ERROR_CONFIG_ERROR;
                }
@@ -1222,6 +1244,29 @@ static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
    return NO_ERROR;
 }
 
+static APIRET ToolCfgGetNextLine (t_pchar *ppLine, t_pint pLineLen)
+{
+   t_pchar pFirstWord;
+   APIRET   rc;
+   int      Eq, RemLineLen;
+   
+   for (;;)  // Stay in loop until non-remark
+   {
+      rc = ToolCfgGetNextLine0 (ppLine, pLineLen);
+      if ((rc != NO_ERROR) || (*ppLine == nullptr) || (*pLineLen == 0))
+         return rc;
+
+      if (**ppLine != KEYCHAR_REMARK)
+      {
+         CFG_CHK_APP (ToolCfgFindFirstWord  (*ppLine,  (const char **)&pFirstWord, *pLineLen, &RemLineLen))
+         CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_REMARK, pFirstWord, RemLineLen, &Eq, nullptr))
+         if (!Eq)
+            return rc;
+      }
+   }
+}
+
+
 /* ----------------------------------- */
 /*  Conversion functions               */
 /* ----------------------------------- */
@@ -1397,7 +1442,7 @@ static APIRET ToolCfgConvertpParamToDouble (t_pchar pParam, t_int ParamLen, t_pd
 
    // switch the 'locale' to accept '.' as decimal point,
    // scan the double value and switch back.
-   pOldLocale = setlocale (LC_NUMERIC, NULL);
+   pOldLocale = setlocale (LC_NUMERIC, nullptr);
    pSavedLocale = strdup (pOldLocale);
    (void) setlocale (LC_NUMERIC, "C");
    rc = sscanf (pParam, "%LG", &LongDoubleVal);
@@ -1549,17 +1594,17 @@ static APIRET ToolCfgConvertpParamToSet (t_pchar pParam, t_int ParamLen, t_pTool
    pSetArray = pCfgDataDesc->pSetArray;
    for (i=0; ; i++)
    {
-      if (pSetArray[i].pSetString == NULL)
+      if (pSetArray[i].pSetString == nullptr)
          break; /* end of array */
       if (ParamLen == (t_int)strlen (pSetArray[i].pSetString))
          if (ToolCfgStrNCmpNoCase (pSetArray[i].pSetString, pParam, ParamLen) == 0)
             break; /* set entry found */
    }
-   if (pSetArray[i].pSetString == NULL)
+   if (pSetArray[i].pSetString == nullptr)
    {
       CFG_CHK_APP (ToolCfgStdErrorHeader (pParam))
       CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Invalid set value."))
-      CFG_CHK_APP (ToolCfgBuildHelp (pCfgDataDesc, NULL, &Len))
+      CFG_CHK_APP (ToolCfgBuildHelp (pCfgDataDesc, nullptr, &Len))
       CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTmp, (t_uint)Len, MEM_OPT_NONE, MemIdCfgHelpBuff))
       CFG_CHK_APP (ToolCfgBuildHelp (pCfgDataDesc, pTmp, &Len))
       CFG_CHK_APP (ToolCfgLogEntry(__FFL__, pTmp))
@@ -1663,7 +1708,7 @@ static APIRET ToolCfgScanTableAnalyseLine (t_pToolCfgTableDesc pCfgTableDesc, t_
    for (i=0; ;i++)
    {
       pCfgDataDesc = &pCfgTableDesc->pDataDescArray[i];
-      if (pCfgDataDesc->pName == NULL)
+      if (pCfgDataDesc->pName == nullptr)
          break; /* Array ends here */
 
       CFG_CHK_APP (ToolCfgFindFirstWord (pFirstWord + ParamLen, (const char **) &pFirstWord, RemLineLen, &RemLineLen))
@@ -1687,14 +1732,14 @@ static APIRET ToolCfgScanTable (t_pchar pActLine, t_int LineLen, t_pToolCfgTable
    long      BaseAddr;
    t_pcchar pErrorText;
    t_char    TmpChar;
-   t_pToolCfgTableDesc pCfgTableDesc = NULL;
+   t_pToolCfgTableDesc pCfgTableDesc = nullptr;
 
    TmpChar = pActLine[LineLen];  /* set end of line to '\0' to ease processing */
    pActLine[LineLen] = '\0';
    CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTableType, 64, MEM_OPT_NONE, MemIdCfgTableType))
    CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTableName, 64, MEM_OPT_NONE, MemIdCfgTableName))
    CFG_CHK_APP (ToolCfgFindFirstWord (pActLine, (const char **) &pFirstWord, LineLen, &RemLineLen))
-   rcs = sscanf (pActLine, "%*s %s %s", pTableType, pTableName);
+   rcs = sscanf (pActLine, "%*s %64s %64s", pTableType, pTableName);
    if (rcs != 2)
    {
       CFG_CHK_APP (ToolCfgStdErrorHeader (pFirstWord))
@@ -1703,12 +1748,12 @@ static APIRET ToolCfgScanTable (t_pchar pActLine, t_int LineLen, t_pToolCfgTable
       return TOOLCFG_ERROR_CONFIG_ERROR;
    }
    Found = FALSE;
-   if (pCfgTableDescArray != NULL)
+   if (pCfgTableDescArray != nullptr)
    {
       for (i=0; ;i++)
       {
          pCfgTableDesc = &pCfgTableDescArray[i];
-         if (pCfgTableDesc->pTableType == NULL) /* End of array reached? */
+         if (pCfgTableDesc->pTableType == nullptr) /* End of array reached? */
             break;
          if (ToolCfgStrCmpNoCase(pCfgTableDesc->pTableType, pTableType) == 0) /* Table type found? */
          {
@@ -1735,16 +1780,16 @@ static APIRET ToolCfgScanTable (t_pchar pActLine, t_int LineLen, t_pToolCfgTable
    for (LineLen=0;;)
    {
       CFG_CHK_APP (ToolCfgGetNextLine (&pActLine, &LineLen))
-      TmpChar = pActLine[LineLen];  /* set end of line to '\0' to ease processing */
-      pActLine[LineLen] = '\0';
-      if (pActLine == NULL)
+      if (pActLine == nullptr)
       {
-         CFG_CHK_APP (ToolCfgStdErrorHeader (NULL))
+         CFG_CHK_APP (ToolCfgStdErrorHeader (nullptr))
          CFG_CHK_APP (ToolCfgLogEntry(__FFL__, "Missing statement %s before end of file.", KEYWORD_TABLEEND))
          return TOOLCFG_ERROR_CONFIG_ERROR;
       }
+      TmpChar = pActLine[LineLen];  // set end of line to '\0' to ease processing
+      pActLine[LineLen] = '\0';
       CFG_CHK_APP (ToolCfgFindFirstWord (pActLine, (const char **) &pFirstWord, LineLen, &RemLineLen))
-      CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_TABLEEND, pFirstWord, RemLineLen, &Eq, NULL))
+      CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_TABLEEND, pFirstWord, RemLineLen, &Eq, nullptr))
       if (Eq)
       {
          if (pCfgTableDesc->pEndFn)
@@ -1760,19 +1805,15 @@ static APIRET ToolCfgScanTable (t_pchar pActLine, t_int LineLen, t_pToolCfgTable
          pActLine[LineLen] = TmpChar;
          break;
       }
-      CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_REMARK, pFirstWord, RemLineLen, &Eq, NULL))
-      if (!Eq)
+      CFG_CHK_APP (ToolCfgScanTableAnalyseLine (pCfgTableDesc, pFirstWord, RemLineLen, BaseAddr, &LineOk))
+      if (LineOk)
       {
-         CFG_CHK_APP (ToolCfgScanTableAnalyseLine (pCfgTableDesc, pFirstWord, RemLineLen, BaseAddr, &LineOk))
-         if (LineOk)
+         CFG_CHK_APP ((*pCfgTableDesc->pSaveAndNextFn) (&BaseAddr, &pErrorText))
+         if (pErrorText)
          {
-            CFG_CHK_APP ((*pCfgTableDesc->pSaveAndNextFn) (&BaseAddr, &pErrorText))
-            if (pErrorText)
-            {
-               CFG_CHK_APP (ToolCfgStdErrorHeader (pActLine))
-               CFG_CHK_APP (ToolCfgLogEntry(__FFL__, pErrorText))
-               return TOOLCFG_ERROR_CONFIG_ERROR;
-            }
+            CFG_CHK_APP (ToolCfgStdErrorHeader (pActLine))
+            CFG_CHK_APP (ToolCfgLogEntry(__FFL__, pErrorText))
+            return TOOLCFG_ERROR_CONFIG_ERROR;
          }
       }
       pActLine[LineLen] = TmpChar;
@@ -1865,8 +1906,8 @@ static APIRET ToolCfgReadParameter (t_pchar pKeyWord, t_pchar pNextChar, t_pTool
 {
    t_pToolCfgDataDesc pCfgDataDesc;
    t_pchar        pFirstParamChar;
-   t_pchar        pErrorText;
-   t_int           Assign;
+   t_pcchar       pErrorText;
+   t_int           Assign=0;
    t_int           ParamLen;
 
    if (pCfgParamDesc)
@@ -1913,7 +1954,9 @@ static APIRET ToolCfgAnalyseEntry (t_pchar pActLine, t_int LineLen,
    t_pToolCfgDataDesc pCfgDataDesc;
    t_int           i;
    t_int           RemLineLen;
-   t_int           Eq, EqDef, EqUnDef;
+   t_int           Eq      = 0;
+   t_int           EqDef   = 0;
+   t_int           EqUnDef = 0;
    t_char          TmpChar;
    t_pchar        pSectionName;
    t_int           SectionNameLen;
@@ -1926,13 +1969,13 @@ static APIRET ToolCfgAnalyseEntry (t_pchar pActLine, t_int LineLen,
    pActLine[LineLen] = '\0';
 
    CFG_CHK_APP (ToolCfgFindFirstWord (pActLine, (const char **) &pKeyWord, LineLen, &RemLineLen))
-   if (pKeyWord == NULL)             // Were there just spaces or tabs in this line ?
+   if (pKeyWord == nullptr)             // Were there just spaces or tabs in this line ?
    {
       pActLine[LineLen] = TmpChar;
       return NO_ERROR;
    }
 
-   CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_TABLESTART, pKeyWord, RemLineLen, &Eq, NULL))
+   CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_TABLESTART, pKeyWord, RemLineLen, &Eq, nullptr))
    if (Eq)
    {
       pActLine[LineLen] = TmpChar;
@@ -1940,13 +1983,6 @@ static APIRET ToolCfgAnalyseEntry (t_pchar pActLine, t_int LineLen,
       return NO_ERROR;
    }
 
-   CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_REMARK, pKeyWord, RemLineLen, &Eq, NULL))
-   if (Eq)
-   {
-      pActLine[LineLen] = TmpChar;
-      return NO_ERROR;
-   }
-
    CFG_CHK_APP    (ToolCfgCompareKeyWord (KEYWORD_DEFINE  , pKeyWord, RemLineLen, &EqDef  , &pSectionName))
    if (!EqDef)
       CFG_CHK_APP (ToolCfgCompareKeyWord (KEYWORD_UNDEFINE, pKeyWord, RemLineLen, &EqUnDef, &pSectionName))
@@ -1954,6 +1990,13 @@ static APIRET ToolCfgAnalyseEntry (t_pchar pActLine, t_int LineLen,
    {
       while (((*pSectionName == ' ') || (*pSectionName == '\t')) && (*pSectionName != '\0'))
          pSectionName++;
+         
+      if (*pSectionName == '=')
+      {
+         pSectionName++;
+         while (((*pSectionName == ' ') || (*pSectionName == '\t')) && (*pSectionName != '\0'))
+            pSectionName++;
+      }
 
       CFG_CHK_APP (ToolCfgCheckIfOnlyOneParam (pSectionName, &SectionNameLen))
       pActLine[LineLen] = TmpChar;
@@ -1987,12 +2030,12 @@ static APIRET ToolCfgAnalyseEntry (t_pchar pActLine, t_int LineLen,
    }
 
    Eq = FALSE;
-   if (pParamDescArray != NULL)
+   if (pParamDescArray != nullptr)
    {
       for (i=0; ; i++)
       {
          pCfgDataDesc = &pParamDescArray[i].DataDesc;
-         if (pCfgDataDesc->pName == NULL)
+         if (pCfgDataDesc->pName == nullptr)
             break;/* end of array */
          CFG_CHK_APP (ToolCfgCompareKeyWord (pCfgDataDesc->pName, pKeyWord, RemLineLen, &Eq, &pNextChar))
          if (Eq)
@@ -2068,8 +2111,8 @@ static APIRET ToolCfgScanSource (t_pcchar pFileName,
    /* initialise first context stack entry */
    ToolCfgLocal.pActCfgContext->BufferLen      = 0;
    ToolCfgLocal.pActCfgContext->ActLineNr      = 0;
-   ToolCfgLocal.pActCfgContext->pActLine       = NULL;
-   ToolCfgLocal.pActCfgContext->pBuffer        = NULL;
+   ToolCfgLocal.pActCfgContext->pActLine       = nullptr;
+   ToolCfgLocal.pActCfgContext->pBuffer        = nullptr;
    ToolCfgLocal.pActCfgContext->SectionNesting = 0;
    ToolCfgLocal.pActCfgContext->State          = CFGSTATE_INCLUDENEWFILE;
    CFG_CHK_APP (ToolCfgCopyName (pFileName   , & ToolCfgLocal.pActCfgContext->FileName         [0] , MAX_FILENAME_LEN   ))
@@ -2096,7 +2139,7 @@ static APIRET ToolCfgCheckInitialisation(t_pToolCfgParamDesc pParamDescArray)
    {
       pCfgParamDesc = &pParamDescArray[i];
       pCfgDataDesc  = &pCfgParamDesc->DataDesc;
-      if (pCfgDataDesc->pName == NULL)
+      if (pCfgDataDesc->pName == nullptr)
          break; /* end of array reached */
       if (( pCfgParamDesc->CfgAssignments   == 0               ) &&
           ( pCfgParamDesc->CmdAssignments   == 0               ) &&
@@ -2118,10 +2161,10 @@ static APIRET ToolCfgLogGetSetName (t_pToolCfgSet pSetArray, t_int SetValue, t_p
 {
    t_int i;
 
-   *ppSetString = NULL;
+   *ppSetString = nullptr;
    for (i=0; ; i++)
    {
-      if (pSetArray[i].pSetString == NULL)
+      if (pSetArray[i].pSetString == nullptr)
          break; /* end of array */
       if (pSetArray[i].SetValue == SetValue)
       {
@@ -2157,7 +2200,7 @@ static APIRET ToolCfgPrintContents (t_pchar pBuff, t_pToolCfgParamDesc pCfgParam
 
          case CFGTYPE_SET:     SetValue = *(int *)pCfgDataDesc->DestAddr;
                                CFG_CHK_APP (ToolCfgLogGetSetName (pCfgDataDesc->pSetArray, SetValue, (const char **) &pSetString))
-                               if (pSetString == NULL)
+                               if (pSetString == nullptr)
                                     sprintf (pBuff, "Set str unknown for value %d", SetValue);
                                else sprintf (pBuff, "%s", pSetString);
             break;
@@ -2184,7 +2227,7 @@ APIRET ToolCfgPrintParamContents (t_pToolCfgParamDesc pParamDescArray, t_pchar p
    {
       pCfgParamDesc = &pParamDescArray[i];
       pCfgDataDesc  = &pCfgParamDesc->DataDesc;
-      if (pCfgDataDesc->pName == NULL)
+      if (pCfgDataDesc->pName == nullptr)
          return TOOLCFG_ERROR_UNKNOWN_PARAMETER;
       if (ToolCfgStrCmpNoCase (pCfgDataDesc->pName, pName) == 0)
       {
@@ -2212,7 +2255,7 @@ APIRET ToolCfgLogConfiguration (t_pToolCfgParamDesc pParamDescArray)
    {
       pCfgParamDesc = &pParamDescArray[i];
       pCfgDataDesc  = &pCfgParamDesc->DataDesc;
-      if (pCfgDataDesc->pName == NULL)
+      if (pCfgDataDesc->pName == nullptr)
          break; /* end of array reached */
       if (( pCfgParamDesc->CfgAssignments   == 0               ) &&
           ( pCfgParamDesc->CmdAssignments   == 0               ) &&
@@ -2244,7 +2287,7 @@ static APIRET ToolCfgResetAssignCounters (t_pToolCfgParamDesc pParamDescArray)
 
    for (i=0; ; i++)
    {
-      if (pParamDescArray[i].DataDesc.pName == NULL)
+      if (pParamDescArray[i].DataDesc.pName == nullptr)
          break; /* end of array reached */
       pParamDescArray[i].CmdAssignments = 0;
       pParamDescArray[i].CfgAssignments = 0;
@@ -2257,12 +2300,12 @@ static APIRET ToolCfgResetAssignCounters (t_pToolCfgParamDesc pParamDescArray)
 /* usually called to get the configuration or log file name.                   */
 APIRET ToolCfgGetCmdLineOption (t_pcchar pParamName, t_pcchar *ppValue)
 {
-   t_int        i, Eq;
-   t_int        LineLen  = 0;
-   t_pchar     pNextChar = NULL;
-   t_ToolCfgContext CfgContext;
+   t_ToolCfgContext  CfgContext;
+   t_int             i, Eq;
+   t_int             LineLen  = 0;
+   t_pchar          pNextChar = nullptr;
 
-   *ppValue = NULL;
+   *ppValue = nullptr;
    Eq = 0;
    for (i=1; i<ToolCfgLocal.argc; i++)
    {
@@ -2283,16 +2326,16 @@ APIRET ToolCfgGetCmdLineOption (t_pcchar pParamName, t_pcchar *ppValue)
    return NO_ERROR;
 }
 
-/* CfgScanConfiguration: API functions to scan the configuration. Two subcalls */
-/* have to be done: Scan the command line and scan the configuration file.     */
+/* CfgScanConfiguration: API function to scan the configuration. Two subcalls */
+/* have to be done: Scan the command line and scan the configuration file.    */
 APIRET ToolCfgScanConfiguration (t_pcchar pFileName, t_pcchar pSectionName,
                              t_ToolCfgParamDesc *pParamDescArray,
                              t_ToolCfgTableDesc *pTableDescArray)
 {
    CFG_CHK_APP (ToolCfgResetAssignCounters (pParamDescArray))
    ToolCfgLocal.Priority = PRIORITY_HIGH;   /* we will now scan the command line params which have high priority */
-   CFG_CHK_APP (ToolCfgScanSource (NULL, pSectionName, pParamDescArray, pTableDescArray))
-   if (pFileName != NULL)
+   CFG_CHK_APP (ToolCfgScanSource (nullptr, pSectionName, pParamDescArray, pTableDescArray))
+   if (pFileName != nullptr)
    {
       ToolCfgLocal.Priority = PRIORITY_LOW;    /* cfg file params with low priority */
       CFG_CHK_APP (ToolCfgScanSource (pFileName, pSectionName, pParamDescArray, pTableDescArray))
@@ -2312,11 +2355,11 @@ static APIRET ToolCfgEnterRemark (FILE *pFile, t_pcchar pRemark, t_int Indent, t
    t_int LineLen;
    t_int i;
 
-   rcp = fprintf (pFile, "\r\n%*s%s %s", Indent, "", KEYWORD_REMARK, pRemark);
+   rcp = fprintf (pFile, "\r\n%*s%c %s", Indent, "", KEYCHAR_REMARK, pRemark);
    if ((rcp > 0) && (LineFlag))
    {
       LineLen = (t_int) strlen (pRemark);
-      rcp = fprintf (pFile, "\r\n%*s%s ", Indent, "", KEYWORD_REMARK);
+      rcp = fprintf (pFile, "\r\n%*s%c ", Indent, "", KEYCHAR_REMARK);
       CFG_CHK_FPRINTF(rcp)
       for (i=0; i<LineLen; i++)
       {
@@ -2351,7 +2394,7 @@ static APIRET ToolCfgBuildTemplate0 (FILE *pFile, t_int CfgParams, t_pint pEntri
       for (i=0; ; i++)
       {
          pCfgDataDesc = &pParamDescArray[i].DataDesc;
-         if (pCfgDataDesc->pName == NULL)
+         if (pCfgDataDesc->pName == nullptr)
             break; /* end of array reached */
 
          Source = (t_ToolCfgAssignment) (pParamDescArray[i].Assign & CFGASN_SOURCE);
@@ -2359,7 +2402,7 @@ static APIRET ToolCfgBuildTemplate0 (FILE *pFile, t_int CfgParams, t_pint pEntri
          if (((Source & CFGASN_CMD) && !(Source & CFGASN_CFG) && (CfgParams == FALSE)) ||
              ((Source & CFGASN_CFG) &&                           (CfgParams == TRUE )))
          {
-            CFG_CHK_APP (ToolCfgBuildHelp (pCfgDataDesc, NULL, &Len))
+            CFG_CHK_APP (ToolCfgBuildHelp (pCfgDataDesc, nullptr, &Len))
             CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTmp1, (t_uint)Len, MEM_OPT_NONE, MemIdCfgHelpBuff))
             CFG_CHK_APP (ToolCfgBuildHelp (pCfgDataDesc, pTmp1, &Len))
             CFG_CHK_FPRINTF (fprintf (pFile, "\r\n   %s", pTmp1))
@@ -2373,9 +2416,9 @@ static APIRET ToolCfgBuildTemplate0 (FILE *pFile, t_int CfgParams, t_pint pEntri
       for (i=0; ; i++)
       {
          pCfgTableDesc = &pTableDescArray[i];
-         if (pCfgTableDesc->pTableType == NULL)
+         if (pCfgTableDesc->pTableType == nullptr)
             break; /* end of array reached */
-         CFG_CHK_APP (ToolCfgBuildTableHelp (pCfgTableDesc, NULL, NULL, &Len))
+         CFG_CHK_APP (ToolCfgBuildTableHelp (pCfgTableDesc, nullptr, nullptr, &Len))
          CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTmp1, (t_uint)Len, MEM_OPT_NONE, MemIdCfgHelpBuff))
          CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTmp2, (t_uint)Len, MEM_OPT_NONE, MemIdCfgHelpBuff))
          CFG_CHK_APP (ToolCfgBuildTableHelp (pCfgTableDesc, pTmp1, pTmp2, &Len))
@@ -2404,9 +2447,9 @@ APIRET ToolCfgBuildTemplate (t_pcchar pFileName, t_pcchar pComment,
    time_t          NowT;
 
    pFile = fopen(pFileName, "wb");
-   if (pFile == NULL)
+   if (pFile == nullptr)
       return TOOLCFG_ERROR_TEMPLATE_OPEN_FAILED;
-   setbuf (pFile, NULL);
+   setbuf (pFile, nullptr);
 
    CFG_CHK_APP (ToolCfgMemAlloc ((void **)&pTmp, TmpLen, MEM_OPT_NONE, MemIdCfgTemplateHeader))
    (void) time (&NowT);
@@ -2453,8 +2496,8 @@ APIRET ToolCfgGetSetString (t_pToolCfgSet pSetArray, int SetValue, const char **
 {
    t_pToolCfgSet pSet;
 
-   *ppSetString = NULL;
-   for (pSet = &pSetArray[0]; pSet->pSetString != NULL; pSet++)
+   *ppSetString = nullptr;
+   for (pSet = &pSetArray[0]; pSet->pSetString != nullptr; pSet++)
    {
       if (pSet->SetValue == SetValue)
       {
@@ -2497,6 +2540,7 @@ APIRET ToolCfgInit (int argc, char *argv[])
    CFG_CHK_APP (TOOL_ERROR_REGISTER_CODE (TOOLCFG_ERROR_TOO_MANY_SECTIONNAMES    ))
    CFG_CHK_APP (TOOL_ERROR_REGISTER_CODE (TOOLCFG_ERROR_SECTIONNAME_NOTFOUND     ))
    CFG_CHK_APP (TOOL_ERROR_REGISTER_CODE (TOOLCFG_ERROR_UNKNOWN_PARAMETER        ))
+   CFG_CHK_APP (TOOL_ERROR_REGISTER_CODE (TOOLCFG_ERROR_UNEXPECTED_EOL          ))
 
    CFG_CHK_APP (MEM_REGISTER_MEMID (MemIdCfgBuffer          ))
    CFG_CHK_APP (MEM_REGISTER_MEMID (MemIdCfgErrorHeader     ))
@@ -2512,7 +2556,7 @@ APIRET ToolCfgInit (int argc, char *argv[])
 
    ToolCfgLocal.argc = argc;
    ToolCfgLocal.argv = argv;
-   ToolCfgLocal.pUserLogFn = NULL;
+   ToolCfgLocal.pUserLogFn = nullptr;
    ToolCfgLocal.MaxIncludeNestingLevel = MAX_CFG_NESTING_LEVEL;
    memset (&ToolCfgLocal.GlobalSectionNameArr[0], 0, sizeof (ToolCfgLocal.GlobalSectionNameArr));
 
